var Protocol = function()
{
	var protocol = {};
	var debug = 0;
	var MINIMAL_DNA_VOLUME = 1.0; //µl
	
	// Page initialization
	protocol.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'protocol')
		{
			Buttons.addClickHandler('print-button', Wizard.goPrint);	

			var submitInfoJsonStr = Data.get('page-data', 'submitinfo');
			protocol.initializeProtocol(submitInfoJsonStr);
		}
	}

	protocol.initializeProtocol = function(submitInfoJsonStr)
	{
		var submitInfo = JSON.parse(submitInfoJsonStr);
		var docMode = submitInfo.docMode;
		var itemInfo = submitInfo.items;
		var list = itemInfo;

		var schema = PoolSchema.getById(Data.get('page-data', 'schema'));

		var view = Data.get('page-data', 'view');
		if (view == 'list')
		{
			protocol.viewAsList(list);
		}
		else
		{
			protocol.viewAsPlate(list, schema)
		}

	}
	
	protocol.viewAsPlate = function(list, schema)
	{
		var rows = Data.get('page-data', 'rows');
		var columns = Data.get('page-data', 'columns');
		Plate.init(rows, columns, schema, WellPainter);
		
		for (var i = 0; i < list.length; i++)
		{
			var lib = list[i];
			var well = lib.bioWell;
			Plate.getWell(well.row, well.column).setExtract(lib);
			// FPB aliquot
			var libFpb = protocol.fetchFpbAliquot(lib);
			var wellFpb = libFpb.bioWell;
			Plate.getWell(wellFpb.row, wellFpb.column).setExtract(libFpb);
		}

		Plate.paint(Plate.getWells());
		PoolSchema.buildPoolTableRow(schema, columns);

		Doc.show('all-protocol');
	}

	protocol.viewAsList = function(list)
	{
		var lastPoolNum = -1;
		for (var i = 0; i < list.length; i++)
		{
			var lib = list[i];
			protocol.libListEntry(lib);
			// FPB aliquot
			var libFpb = protocol.fetchFpbAliquot(lib);
			protocol.libListEntry(libFpb);
		}

		Doc.show('all-protocol');
	}

	protocol.libListEntry = function(lib)
	{
		var well = lib.bioWell;
	
		var idSuffix = well.column + '.' + well.row;
		Doc.removeClass('row.'+idSuffix, 'empty');
		Doc.element('lib.'+idSuffix).innerHTML = Strings.encodeTags(lib.name);
		// Index 1 & 2 primers
		var well = lib.bioWell;
		var row = well.row + 1;
		var column = well.column + 1;
		var index2 = 'A50' + row;
		var index1 = 'A70' + column;
		if (column > 9)
		{
			index1 = 'A7' + column;
		}
		Doc.element('barcode.'+idSuffix).innerHTML = Strings.encodeTags(index2 + ', ' + index1);
		// DNA extract
		var dna = lib.info;
		if (dna)
		{
			var volDnaInSolution = Numbers.formatNumber(dna.volDnaInSolution, 2);
			var waterInSolution = Numbers.formatNumber(dna.waterInSolution, 2);
			if (dna.volDnaInSolution < MINIMAL_DNA_VOLUME)
			{
				Doc.element('dilution.'+idSuffix).innerHTML = '<span class="lowvolume">' + volDnaInSolution + ' + ' + waterInSolution + 'µl</span>';
			}
			else
			{
				Doc.element('dilution.'+idSuffix).innerHTML = volDnaInSolution + ' + ' + waterInSolution + 'µl';
			}
		}
			
		if (lib.remarks)
		{
			Doc.element('remarks.'+idSuffix).innerHTML = lib.remarks.join('; ');
		}
	}
	
	protocol.fetchFpbAliquot = function(lib)
	{
		// FPA aliquot
		var well = lib.bioWell;
		var fpaName = lib.name;
		var fpaInfo = lib.info;
		// FPB aliquot
		var row = well.row;
		var column = well.column;
		if (column < 6)
		{
			column += 6;
		}
		var wellFpb = {};
		wellFpb.row = row;
		wellFpb.column = column;
		var aliquotExtensionIndex = fpaName.indexOf('.fpa');
		var fpbName = fpaName.substring(0, aliquotExtensionIndex) + ".fpb";
		var libFpb = {};
		libFpb.bioWell = wellFpb;
		libFpb.name = fpbName;
		// Clone of FPA info is OK for library preparation plate protocol
		libFpb.info = fpaInfo;
		return libFpb;
	}

	return protocol;
}();

Doc.onLoad(Protocol.initPage);


var WellPainter = function()
{
	var MINIMAL_DNA_VOLUME = 1.0; //µl
	var painter = {};
	
	painter.getClassNameForWell = function(well, schema)
	{
		var cls = '';
		if (well.column < 6)
		{
			cls = 'bg-green';
		}
		else
		{
			cls = 'bg-blue';
		}
		return cls;
	}
	
	painter.getWellText = function(well, schema)
	{
		var text = '';
		var lib = well.extract;
		if (lib)
		{
			var name = lib.name;
			var aliquotName = painter.fetchPlateViewAliquotName(name);
			text += '<div class="lib">'+Strings.encodeTags(aliquotName)+'</div>';
			// DNA extract
			var dna = lib.info;
			if (dna)
			{
				var volDnaInSolution = Numbers.formatNumber(dna.volDnaInSolution, 2);
				var waterInSolution = Numbers.formatNumber(dna.waterInSolution, 2);
				if (dna.volDnaInSolution < MINIMAL_DNA_VOLUME)
				{
					text += '<div class="lib"><span class="lowvolume">'+volDnaInSolution+' + '+waterInSolution+'µl</span></div>';
				}
				else
				{
					text += '<div class="lib">'+volDnaInSolution+' + '+waterInSolution+'µl</div>';
				}
			}
			// Index 1 & 2 primers
			var well = lib.bioWell;
			var row = well.row + 1;
			var column = well.column + 1;
			var index2 = 'A50' + row;
			var index1 = 'A70' + column;
			if (column > 9)
			{
				index1 = 'A7' + column;
			}
			text += '<div class="lib">'+index2+'</div>';
			text += '<div class="lib">'+index1+'</div>';
		}
		return text;
	}

	painter.fetchPlateViewAliquotName = function(name)
	{
		// Concatenate parts before first dot and after last dot and convert to uppercase
		var extIndex = name.indexOf('.fp');
		var origBaseName = name.substring(0, extIndex);
		var origExtName = name.substring(extIndex);
		var baseName = origBaseName.toUpperCase();
		var dotIndex = baseName.indexOf('.');
		baseName = baseName.substring(0, dotIndex);
		var extName = 'FPA';
		if (origExtName == '.fpb')
		{
			extName = 'FPB';
		}
		var aliquotName = baseName + extName;
		return aliquotName;
	}

	return painter;
}();
