var ExtractionProtocol = function()
{
	var ep = {};
	var debug = 0;
	
	ep.TITLE = 'MeLuDI';
	var COLORS = ['blue', 'green', 'yellow', 'red'];
	var nextColor = 0;
	var meludiVersion;

	// Page initialization
	ep.initPage = function()
	{
		Buttons.addClickHandler('print-button', Wizard.goPrint);
		var numItems = Data.get('page-data', 'numitems');
		var submitInfoJsonStr = Data.get('page-data', 'submitinfo');
		meludiVersion = Data.get('page-data', 'meludiversion');
		ep.initializeProtocol(submitInfoJsonStr);
	}

	ep.initializeProtocol = function(submitInfoJsonStr)
	{
		var frm = document.forms['meludi'];

		var submitInfo = JSON.parse(submitInfoJsonStr);
		var docMode = submitInfo.docMode;
		var itemInfo = submitInfo.items;
		var startListName = submitInfo.startListName;
		var isolationDate = submitInfo.isolationDate;
		var extractionOperator = submitInfo.extractionOperator;

		// Specimen reagents
		var xyleneSpecimen = submitInfo.xyleneSpecimen;
		var etoh995Specimen = submitInfo.etoh995Specimen;
		var bufferPkdSpecimen = submitInfo.bufferPkdSpecimen;
		var bufferAtlSpecimen = submitInfo.bufferAtlSpecimen;
		var proteinaseKSpecimen = submitInfo.proteinaseKSpecimen;
		var rnaseASpecimen = submitInfo.rnaseASpecimen;

		// QIAcube
		var qiacubeDate = submitInfo.qiacubeDate;
		var qiacubeOperator = submitInfo.qiacubeOperator;
		var qiacubeRunNo = submitInfo.qiacubeRunNo;
		var allPrepFfpeKit = submitInfo.allPrepFfpeKit;
		var etoh995 = submitInfo.etoh995;
		var bufferRlt = submitInfo.bufferRlt;
		var bufferFrn = submitInfo.bufferFrn;
		var bufferRpe = submitInfo.bufferRpe;
		var bufferAl = submitInfo.bufferAl;
		var bufferAw1 = submitInfo.bufferAw1;
		var bufferAw2 = submitInfo.bufferAw2;
		var bufferAte = submitInfo.bufferAte;
		var bufferRdd = submitInfo.bufferRdd;
		var rnaseFreeWater = submitInfo.rnaseFreeWater;
		var dnaseMix = submitInfo.dnaseMix;
		var rneasyMinEluteSpinColumn = submitInfo.rneasyMinEluteSpinColumn;
		var qiaampMinEluteSpinColumn = submitInfo.qiaampMinEluteSpinColumn;

		// Quality control reagents
		var kapaSybrFast = submitInfo.kapaSybrFast;
		var qualityControlPrimersQcpRgt = submitInfo.qualityControlPrimersQcpRgt;
		var qualityControlTemplateQct = submitInfo.qualityControlTemplateQct;
		var qubitDnaHighSens = submitInfo.qubitDnaHighSens;
		var qubitDnaBroadRange = submitInfo.qubitDnaBroadRange;

		// Various
		var dnaDefaultVolume = submitInfo.dnaDefaultVolume;
		var dnaProtocolName = submitInfo.dnaProtocolName;
		var rnaProtocolName = submitInfo.rnaProtocolName;
		var protocolName = submitInfo.extractionProtocolName;
		//var protocolVersionNo = submitInfo.protocolVersionNo;
		var protocolVersionNo = '1.0';
		var protocolPreviewFlag = submitInfo.protocolPreviewFlag;
		var numQiacubeItems = itemInfo.length;

		// Header data
		var docModeText = 'Protocol';
		if (docMode && docMode == 'report')
		{
			docModeText = 'Report';
		}
		if (protocolVersionNo == null || protocolVersionNo == '')
		{
			protocolVersionNo = '?';
		}
		var protocolPreviewText = '';
		if (protocolPreviewFlag != null && protocolPreviewFlag == 'true')
		{
			protocolPreviewText = ' <i>(preview)</i>';
		}
/*
		var headerText = 'Lab Tracking ' + docModeText + ' for Allprep isolation v' + protocolVersionNo + protocolPreviewText + ' <span class="meludi">' + ExtractionProtocol.TITLE + ' ' + meludiVersion + '</span>';
*/
		var headerText = 'Lab Tracking ' + docModeText + ' for FFPE extraction v' + protocolVersionNo + protocolPreviewText + ' <span class="meludi">' + ExtractionProtocol.TITLE + ' ' + meludiVersion + '</span>';
		Doc.element('allprepLabTrackingProtocolHeader').innerHTML = headerText;
/*
		Doc.element('allprepLabTrackingProtocolHeader2').innerHTML = protocolName;
*/

		Doc.element('startlistid').innerHTML = Strings.encodeTags(startListName);

		Doc.element('isolationdate').innerHTML = Strings.encodeTags(isolationDate);
		Doc.element('operator').innerHTML = Strings.encodeTags(extractionOperator);
		Doc.element('extractionprotocol').innerHTML = Strings.encodeTags(protocolName);

		// Specimen reagents
		Doc.element('xyleneSpecimen').innerHTML = Strings.encodeTags(xyleneSpecimen);
		Doc.element('etoh995Specimen').innerHTML = Strings.encodeTags(etoh995Specimen);
		Doc.element('bufferPkdSpecimen').innerHTML = Strings.encodeTags(bufferPkdSpecimen);
		Doc.element('bufferAtlSpecimen').innerHTML = Strings.encodeTags(bufferAtlSpecimen);
		Doc.element('proteinaseKSpecimen').innerHTML = Strings.encodeTags(proteinaseKSpecimen);
		Doc.element('rnaseASpecimen').innerHTML = Strings.encodeTags(rnaseASpecimen);

		// QIAcube
		Doc.element('qiacubeDate').innerHTML = Strings.encodeTags(qiacubeDate);
		//Doc.element('qiacubeRunNo').innerHTML = Strings.encodeTags(qiacubeRunNo);
		Doc.element('qiacubeOperator').innerHTML = Strings.encodeTags(qiacubeOperator);
		Doc.element('allPrepFfpeKit').innerHTML = Strings.encodeTags(allPrepFfpeKit);
		Doc.element('etoh995').innerHTML = Strings.encodeTags(etoh995);
		Doc.element('bufferRlt').innerHTML = Strings.encodeTags(bufferRlt);
		Doc.element('bufferFrn').innerHTML = Strings.encodeTags(bufferFrn);
		Doc.element('bufferRpe').innerHTML = Strings.encodeTags(bufferRpe);
		Doc.element('bufferAl').innerHTML = Strings.encodeTags(bufferAl);
		Doc.element('bufferAw1').innerHTML = Strings.encodeTags(bufferAw1);
		Doc.element('bufferAw2').innerHTML = Strings.encodeTags(bufferAw2);
		Doc.element('bufferAte').innerHTML = Strings.encodeTags(bufferAte);
		Doc.element('bufferRdd').innerHTML = Strings.encodeTags(bufferRdd);
		Doc.element('rnaseFreeWater').innerHTML = Strings.encodeTags(rnaseFreeWater);
		Doc.element('dnaseMix').innerHTML = Strings.encodeTags(dnaseMix);
		Doc.element('rneasyMinEluteSpinColumn').innerHTML = Strings.encodeTags(rneasyMinEluteSpinColumn);
		Doc.element('qiaampMinEluteSpinColumn').innerHTML = Strings.encodeTags(qiaampMinEluteSpinColumn);

		// Quality control reagents
		Doc.element('kapaSybrFast').innerHTML = Strings.encodeTags(kapaSybrFast);
		Doc.element('qualityControlPrimersQcpRgt').innerHTML = Strings.encodeTags(qualityControlPrimersQcpRgt);
		Doc.element('qualityControlTemplateQct').innerHTML = Strings.encodeTags(qualityControlTemplateQct);
/*
		Doc.element('qubitDnaHighSens').innerHTML = Strings.encodeTags(qubitDnaHighSens);
*/
		Doc.element('qubitDnaBroadRange').innerHTML = Strings.encodeTags(qubitDnaBroadRange);


		//Doc.element('ffpeDnaRnaKitLotNo').innerHTML = Strings.encodeTags(allPrepFfpeKit);
		//Doc.element('proteinaseKLotNo').innerHTML = Strings.encodeTags(proteinaseKSpecimen);
		//Doc.element('dnaseMixLotNo').innerHTML = Strings.encodeTags(dnaseMix);
		//Doc.element('kapaSybrFastLotNo').innerHTML = Strings.encodeTags(kapaSybrFast);
		//Doc.element('qualityControlPrimersQcpRgtNo').innerHTML = Strings.encodeTags(qualityControlPrimersQcpRgt);
		//Doc.element('qualityControlTemplateQctLotNo').innerHTML = Strings.encodeTags(qualityControlTemplateQct);
		//Doc.element('qubitDnaHighSensLotNo').innerHTML = Strings.encodeTags(qubitDnaHighSens);
		//Doc.element('qubitDnaBroadRangeLotNo').innerHTML = Strings.encodeTags(qubitDnaBroadRange);
/*
		if (dnaDefaultVolume)
		{
			Doc.element('dnaElutionVolume').innerHTML = ep.encodeFloatTags(dnaDefaultVolume) + ' µl';
		}
*/

		// Item data
		var accumItemNo = 0;
		accumItemNo = tableRowsForItems(accumItemNo, itemInfo, 'Item', 'item');
		accumItemNo = tableRowForOptionalCounterbalanceItem(accumItemNo, itemInfo, 'Item', 'item');

		Doc.show('all-protocol');
	}
	
	function tableRowsForItems(
		accumItemNo,
		itemJsonArr,
		itemType,
		itemTypeVarName)
	{
		// Table item row
		var html = '';
		if (itemJsonArr && itemJsonArr.length > 0)
		{
			for (var i = 0; i < itemJsonArr.length; i++)
			{
				var item = itemJsonArr[i];
/*
				var storageId = '-';
				var storagePosition = '-';
*/
				var qiacubePosition = '-';
				var platePos = '-';
				var rnaStorageBox = '-';
				var rnaConc = null;
				var dnaStorageBox = '-';
				var dnaConc = null;
				var dnaDeltaCt = null;
				var specimenComment = null;
				var rnaComment = null;
				var dnaComment = null;
				var comment = null;
				if (item)
				{
/*
					storageId = ep.asBioPlateName(item.bioWell);
					storagePosition = ep.asPureBioPlateLocation(item.bioWell);
*/
					qiacubePosition = item.qiacubePosition;

					// RNA extract from specimen
					var rna = item.rna;
					if (rna)
					{
						//platePos = rna.well;
						platePos = rna.nanoDropWell;
						if (rna.bioWell)
						{
							rnaStorageBox = Strings.encodeTags(item.rna.bioWell.bioPlate.name + '[' + item.rna.bioWell.location + ']');
						}
						rnaConc = rna.ndConc;
						rnaComment = rna.comment;
					}

					// DNA extract from specimen
					var dna = item.dna;
					if (dna)
					{
						if (dna.bioWell)
						{
							dnaStorageBox = Strings.encodeTags(item.dna.bioWell.bioPlate.name + '[' + item.dna.bioWell.location + ']');
						}
						dnaConc = dna.qubitConc;
						dnaDeltaCt = dna.deltaCt;
						dnaComment = dna.comment;
					}

					// Input RNA item storage location should be displayed in the RNA extract storage location column
					if (ep.isRna(item) && item.bioWell != null)
					{
						rnaStorageBox = Strings.encodeTags(item.bioWell.bioPlate.name + '[' + item.bioWell.location + ']');
					}

					// Input DNA item storage location should be displayed in the DNA extract storage location column
					if (ep.isDna(item) && item.bioWell != null)
					{
						dnaStorageBox = Strings.encodeTags(item.bioWell.bioPlate.name + '[' + item.bioWell.location + ']');
					}
					var specimenComment = item.comment;
				}
				if (item.operatorDeliveryComment)
				{
					comment = item.operatorDeliveryComment;
				}
				if (specimenComment)
				{
					if (comment)
					{
						comment += ' ';
					}
					else
					{
						comment = '';
					}
					comment += 'Specimen: ' + specimenComment;
				}
				if (rnaComment)
				{
					if (comment)
					{
						comment += ' ';
					}
					else
					{
						comment = '';
					}
					comment += 'RNA: ' + rnaComment;
				}
				if (dnaComment)
				{
					if (comment)
					{
						comment += ' ';
					}
					else
					{
						comment = '';
					}
					comment += 'DNA: ' + dnaComment;
				}
				//alert("tableRowsForItems(): i = " + i + " itemType = " + itemType + " name = " + item.name + " qiacubePosition = " + qiacubePosition + " platePos = " + platePos + " rnaConc = " + rnaConc + " dnaConc = " + dnaConc + " comment = " + comment);
				// Create HTML code for table column with checkbox for marking destroyed items
				// var selectedCheckboxHtml = '<td><b><div class=\"colorbox\"></div>Selected </b></td>';
				// Insert item data in table columns
				Doc.element('sample.'+accumItemNo).innerHTML = Strings.encodeTags(item.name);
/*
				Doc.element('storageId.'+accumItemNo).innerHTML = Strings.encodeTags(storageId);
				Doc.element('storagePosition.'+accumItemNo).innerHTML = Strings.encodeTags(storagePosition);
*/
				Doc.element('qiacubePosition.'+accumItemNo).innerHTML = Strings.encodeTags(qiacubePosition);
				Doc.element('nanodropPlatePosition.'+accumItemNo).innerHTML = Strings.encodeTags(platePos);
				Doc.element('rnaStorageBox.'+accumItemNo).innerHTML = Strings.encodeTags(rnaStorageBox);
				Doc.element('rnaConc.'+accumItemNo).innerHTML = ep.encodeFloatTags(rnaConc);
				Doc.element('dnaStorageBox.'+accumItemNo).innerHTML = Strings.encodeTags(dnaStorageBox);
				Doc.element('dnaConc.'+accumItemNo).innerHTML = ep.encodeFloatTags(dnaConc);
				Doc.element('deltaCt.'+accumItemNo).innerHTML = ep.encodeFloatTags(dnaDeltaCt);
				Doc.element('remark.'+accumItemNo).innerHTML = Strings.encodeTags(comment);
				
				// Increment accumulated item number
				accumItemNo++;
			}
		}
		return accumItemNo;
	}

	function tableRowForOptionalCounterbalanceItem(
		accumItemNo,
		itemJsonArr,
		itemType,
		itemTypeVarName)
	{
		// Find number of QIAcube items
		var lastQiacubeItem;
		var numQiacubeItems = 0;
		if (itemJsonArr && itemJsonArr.length > 0)
		{
			for (var i = 0; i < itemJsonArr.length; i++)
			{
				var item = itemJsonArr[i];
				if (item)
				{
					var qiacubePosition = item.qiacubePosition;
					if (qiacubePosition != null && qiacubePosition != '' && qiacubePosition != '-')
					{
						lastQiacubeItem = item;
						numQiacubeItems++;
					}
				}
			}
		}
		if (numQiacubeItems == 1 || numQiacubeItems == 11 || numQiacubeItems == 23)
		{
			// Table item row
			var html = '';
			var name = 'COUNTER-BALANCE';
			var qiacubePosition = '-';
			var comment = 'Reminder to use counterbalance';
			var lastQiacubePos = lastQiacubeItem.qiacubePosition;
			var lastQiacubePrefix = lastQiacubePos.substring(0,1);
			if (numQiacubeItems == 1)
			{
				qiacubePosition = lastQiacubePrefix + '7';
			}
			else if (numQiacubeItems == 11)
			{
				qiacubePosition = lastQiacubePrefix + '12';
			}
			else if (numQiacubeItems == 23)
			{
				qiacubePosition = lastQiacubePrefix + '12';
			}
			// Create HTML code for table column with checkbox for marking destroyed items
			// Insert item data in table columns
			Doc.element('sample.'+accumItemNo).innerHTML = Strings.encodeTags(name);
			Doc.element('qiacubePosition.'+accumItemNo).innerHTML = Strings.encodeTags(qiacubePosition);
			Doc.element('nanodropPlatePosition.'+accumItemNo).innerHTML = Strings.encodeTags('-');
			Doc.element('rnaStorageBox.'+accumItemNo).innerHTML = Strings.encodeTags('-');
			Doc.element('rnaConc.'+accumItemNo).innerHTML = Strings.encodeTags('-');
			Doc.element('dnaStorageBox.'+accumItemNo).innerHTML = Strings.encodeTags('-');
			Doc.element('dnaConc.'+accumItemNo).innerHTML = Strings.encodeTags('-');
			Doc.element('deltaCt.'+accumItemNo).innerHTML = Strings.encodeTags('-');
			Doc.element('remark.'+accumItemNo).innerHTML = Strings.encodeTags(comment);
				
			// Increment accumulated item number
			accumItemNo++;
		}
		// Table item row
		return accumItemNo;
	}

	ep.isRna = function(item)
	{
		var isRna = false;
		if (item != null)
		{
			var name = item.name;
			if (name != null)
			{
				// Check if item name ends with ".r"
				if (name.indexOf('.r') == (name.length - 2))
				{
					isRna = true;
				}
			}
		}
		return isRna;
	}

	ep.isDna = function(item)
	{
		var isDna = false;
		if (item != null)
		{
			var name = item.name;
			if (name != null)
			{
				// Check if item name ends with ".d"
				if (name.indexOf('.d') == (name.length - 2))
				{
					isDna = true;
				}
			}
		}
		return isDna;
	}

	ep.encodeFloatTags = function(value)
	{
		var encodedValue = value;
		if (!value)
		{
			encodedValue = Strings.encodeTags(value);
		}
		return encodedValue;
	}

	// A biowell as plate name
	ep.asBioPlateName = function(well)
	{
		if (!well) return '';
		var plate = well.bioPlate;
		if (!plate) return '';
		var text = plate.name;
		return text;
	}

	// A biowell as the location
	ep.asPureBioPlateLocation = function(well)
	{
		if (!well) return '';
		var plate = well.bioPlate;
		var text = well.location;
		if (plate.storage)
		{
			var storage = plate.storage;
			var tmp = [];
			if (storage.name) tmp[tmp.length] = storage.name;
			if (storage.section) tmp[tmp.length] = 'section: ' +storage.section;
			if (storage.tray) tmp[tmp.length] = 'tray: ' +storage.tray;
			if (storage.position) tmp[tmp.length] = 'position: ' +storage.position;
			text = text + ' ' + tmp.join('; ');
		}
		return text;
	}

	return ep;
}();

Doc.onLoad(ExtractionProtocol.initPage);

